unit super;

//TSuperList 1.7 - by David Koretzky
//Spanware Inc., copyright 1999

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, DsgnIntf;

type
  TListBoxAlignment = (lbBottom, lbLeft ,lbRight, lbTop);    //added all four alignments
  TEnabledButton = (Plus, Minus);
  FEnabledButtons = set of TEnabledButton;

type
  TSuperList = class(TCustomControl)
  private
    //the list box
    FListBox : TListBox;
    FListBoxAlignment : TListBoxAlignment;
    FListBoxChanged : Boolean;

    //panel for holding the buttons
    FPanel : TPanel;

    //the add and delete buttons
    FAddButton : TSpeedButton;
    FDeleteButton : TSpeedButton;

    //the events
    FOnAddButtonClick: TNotifyEvent;
    FOnDeleteButtonClick: TNotifyEvent;
    FOnListBoxDoubleClick : TNotifyEvent;
    FOnListBoxKeyDown : TNotifyEvent;

    FAddString : string;
    FAddCaption : string;
    FEditString : string;
    FEditCaption : string;

    FMaxLength : Integer;  //max length of string in listbox
    FMaxCount : Integer; //max number of entries allowed in listbox
    FAllowBlanks : boolean; //are blanks allowed in the listbox
    FUseKeyMaps : boolean; //use default key strokes for adding, editing and deleting???

    FDialogBox : TComponent;

    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure AddBtnClick(Sender : TObject);
    procedure DeleteBtnClick(Sender : TObject);
    procedure ListBoxDoubleClick(Sender : TObject);
    procedure ListBoxKeyDown(Sender : TObject; var Key : word; shift : TShiftState);

    procedure CreateButtons;
    procedure CreatePanel;
    procedure EnableButtons;

    procedure SetAlignment(value : TListBoxAlignment);

    function GetListBoxFont : TFont;
    procedure SetListBoxFont(value : TFont);

    function GetListBoxItems : TStrings;
    procedure SetListBoxItems(value : TStrings);

    function GetListBoxItemIndex : Integer;
    procedure SetListBoxItemIndex(value : integer);

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Delete;
    function GetCurrentString : String;
    property ListBoxItemIndex : Integer read GetListBoxItemIndex write SetListBoxItemIndex;
    property ListBox : TListBox read FListBox write FListBox;
    property ListBoxChanged : boolean read FListBoxChanged write FListBoxChanged default false;
    procedure AddItems(value : TStrings);
  published
    property AddPrompt : string read FAddString write FAddString;
    property AddCaption : string read FAddCaption write FAddCaption;
    property Align;
    property AllowBlanks : boolean read FAllowBlanks write FAllowBlanks default false;
    property EditPrompt : string read FEditString write FEditString;
    property EditCaption : string read FEditCaption write FEditCaption;
    property ListBoxAlignment : TListBoxAlignment read FListBoxAlignment write SetAlignment default lbBottom;
    property ListBoxFont : TFont read GetListBoxFont write SetListBoxFont;
    property ListBoxItems : TStrings read GetListBoxItems write SetListBoxItems;
    property MaxLength : Integer read FMaxLength write FMaxLength default 0;
    property MaxCount : Integer read FMaxCount write FMaxCount default 0;
    property OnAddClick: TNotifyEvent read FOnAddButtonClick write FOnAddButtonClick;
    property OnDeleteClick: TNotifyEvent read FOnDeleteButtonClick write FOnDeleteButtonClick;
    property OnListBoxDoubleClick : TNotifyEvent read FOnListBoxDoubleClick write FOnListBoxDoubleClick;
    property OnListBoxKeyDown : TNotifyEvent read FOnListBoxKeyDown write FOnListBoxKeyDown;
    property UseDefaultKeyMaps : boolean read FUsekeyMaps write FUseKeyMaps default false;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TSuperList]);
end;

constructor TSuperList.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Width := 250;
  Height := 200;
  ControlStyle := ControlStyle + [csAcceptsControls];

  FListBox := TListBox.Create(Self);
  FListBox.Parent := Self;
  FListBox.Visible := true;
  FListBox.Width := 250; FListBox.Height := 170;
  FListBox.Top := 20; FListBox.Left := 0;
  FListBox.Align := alClient;
  FListBox.OnDblClick := ListBoxDoubleClick;
  FListBox.OnKeyDown := ListBoxKeyDown;

  CreatePanel;
  CreateButtons;

  FAddCaption := 'Input';
  FAddString := 'Enter New Value';
  FEditCaption := 'Edit';
  FEditString := 'Edit Value';

end;


procedure TSuperList.CreatePanel;
begin
  FPanel := TPanel.Create(Self);
  FPanel.Parent := Self;
  FPanel.Visible := true;
  FPanel.Align := alTop;
  FPanel.Caption := '';
  FPanel.BevelInner := bvLowered;
  FPanel.BevelOuter := bvLowered;
  FPanel.Height := 35;
end;

procedure TSuperList.CreateButtons;
begin
  FaddButton := TSpeedButton.Create(Self);
  FAddButton.Width := 25;
  FAddButton.Height := 25;
  FaddButton.Font.Name := 'Arial Bold';
  FaddButton.Font.Size := 20;
  FaddButton.Caption := '+';
  FaddButton.Parent := FPanel;
  FaddButton.Top := 5; FaddButton.Left := Self.Width - 60;
  FaddButton.OnClick := AddBtnClick;

  FdeleteButton := TSpeedButton.Create(Self);
  FdeleteButton.Width := 25;
  FDeleteButton.Height := 25;
  FdeleteButton.Font.Name := 'Arial Bold';
  FdeleteButton.Font.Size := 20;
  FdeleteButton.Caption := '-';
  FdeleteButton.Parent := FPanel;
  FdeleteButton.Top := 5; FdeleteButton.Left := Self.Width - 30;
  FDeleteButton.OnClick := DeleteBtnClick;
end;

destructor TSuperList.Destroy;
begin
  FListBox.Free;
  FaddButton.Free;
  FdeleteButton.Free;
  FPanel.Free;
  inherited;
end;

procedure TSuperList.WMSize(var Message: TWMSize);
var
  w, h: Integer;
begin
  inherited;

  w := Width;
  h := Height;

  if (w < 75) then Width := 75;
  if (h < 75) then Height := 75;

  case FListBoxAlignment of
    lbBottom, lbTop:
      begin
        FPanel.Height := 35;
        FaddButton.Left := Self.Width - 60;
        FaddButton.Top := 5;
        FdeleteButton.Left := Self.Width - 30;
        FdeleteButton.Top := 5;
      end;
    lbLeft, lbRight:
      begin
        FPanel.Width := 35;
        FAddButton.Left := 5;
        FDeleteButton.Left := 5;
        FAddButton.Top := FPanel.Top + 5;
        FDeleteButton.Top := FPanel.Top + 35;
      end;
  end;

end;

procedure TSuperList.AddBtnClick(Sender: TObject);
var s : string;
begin
  s := InputBox(FAddCaption, FAddString, '');
    if (Length(s) <= FMaxLength ) or (FMaxLength <= 0) then
      if (FAllowBlanks) or ((not FAllowBlanks) and (Length(s) > 0)) then
      begin
        FListBox.Items.Add(s);
        FListBoxChanged := true;
      end;
  if Assigned(FOnAddButtonClick) then FOnAddButtonClick(Self);
  EnableButtons;
end;

procedure TSuperList.DeleteBtnClick(Sender : TObject);
begin
   if (FListBox.ItemIndex > -1) then
   begin
    FListBox.Items.Delete( FListBox.ItemIndex );
    FListBoxChanged := true;
   end;
   if Assigned(FOnDeleteButtonClick) then FOnDeleteButtonClick(Self);
   EnableButtons;
end;

procedure TSuperList.EnableButtons;
begin
   FAddButton.Enabled := (FListBox.Items.Count < FMaxCount) or  (FMaxCount <= 0);
   FDeleteButton.Enabled := (FListBox.Items.Count > 0);
end;

procedure TSuperList.ListBoxDoubleClick(Sender : TObject);
var s : string;
begin
  if (FListBox.ItemIndex < 0) then
    exit;

  s := InputBox(FEditCaption, FEditString, FListBox.Items[FListBox.ItemIndex]);
  if ( s <> FListBox.Items[FListBox.ItemIndex] ) then
  begin
    if (Length(s) < FMaxLength) or (FMaxLength <= 0) then
      if (FAllowBlanks) or ((not FAllowBlanks) and (Length(s) > 0)) then
      begin
        FListBox.Items [FListBox.ItemIndex] := s;
        FListBoxChanged := true;
      end;
    if Assigned(FOnListBoxDoubleClick) then FOnListBoxDoubleClick(Self);
   end;
end;

procedure TSuperList.SetAlignment(value : TListBoxAlignment );
begin
  FListBoxAlignment := value;
  FListBox.Align := alClient;
  case value of
    lbLeft: FPanel.Align := alRight;
    lbTop:  FPanel.Align := alBottom;
    lbRight : FPanel.Align := alLeft;
    lbBottom : FPanel.Align := alTop;
  end;

  SendMessage(Self.Handle, WM_SIZE, 0, 0); //call resize to realign everything
end;

function TSuperList.GetListBoxFont : TFont;
begin
  result := FListBox.Font;
end;

procedure TSuperList.SetListBoxFont(value : TFont);
begin
  FListBox.Font.Assign(value);
end;

function TSuperList.GetListBoxItems : TStrings;
begin
  result := FListBox.Items;
end;

procedure TSuperList.SetListBoxItems(value : TStrings);
begin
  FListBox.Items.Assign(value);
end;

procedure TSuperList.SetListBoxItemIndex(value : integer);
begin
  FListBox.ItemIndex := value;
end;

function TSuperList.GetListBoxItemIndex : integer;
begin
  result := FListBox.ItemIndex;
end;

procedure TSuperList.Delete;
begin
  if (FListBox.ItemIndex > -1) then
  begin
    FListBox.Items.Delete(FListBox.ItemIndex);
    FListBoxChanged := true;
  end;
end;

function TSuperList.GetCurrentString : String;
begin
  if (FListBox.ItemIndex > -1) then
    result := FListBox.Items[ FListBox.ItemIndex]
  else
    result := '';
end;


procedure TSuperList.ListBoxKeyDown(Sender : TObject; var Key : word; shift : TShiftState);
begin
  if (FUseKeyMaps) then
  begin
    if (ssCtrl in shift) then
    case key of
      ord('a'), ord('A') :  AddBtnClick(Sender);
      ord('e'), ord('E') :  ListBoxDoubleClick(Sender);
      ord('d'), ord('D') :  DeleteBtnClick(Sender);
    end;

    if (key = 13) then
      ListBoxDoubleClick(Sender);

  end;

  if Assigned(FOnListBoxKeyDown) then
    FOnListBoxKeyDown(Sender);
end;

procedure TSuperList.AddItems(value : TStrings);
begin
  FListBox.Items.AddStrings(value);
end;

end.
